package com.xintong.visualinspection.service;

import java.util.List;

import com.xintong.visualinspection.bean.Task;
import com.xintong.visualinspection.bean.TaskStatus;
import com.xintong.visualinspection.bean.User;
import com.xintong.visualinspection.bean.UserClassCount;
import com.xintong.visualinspection.pojo.TaskListByDeptAppeal;

/**
 * 
 *
 * ConstantService
 * 
 * tiger
 * tiger
 * 2017年5月4日 上午10:41:40
 * 
 * @version 1.0.0
 *
 */
public interface TaskService {
    public List<Task> getAll();
    public Task getById(Long id);
    public List<Task> getTaskList(Task task);
    public void insert(Task task);
    public void update(Task task);
    public void delete(Long id);
    public List<TaskStatus> getTaskStatusList(Long id);
    public void dispatch(Task task);
    public void dispatchById(Task task);
    public void insertStatus(TaskStatus taskStatus);
    public void insertStatusBatch(List<TaskStatus> taskStatusList);
    public void deleteInvalidTaskCount(Long task_id);
    public void updateInvalidTaskCount(Task task);
    public List<UserClassCount> getUserCount(Task task);
    public List<Task> getTaskListByDept(TaskListByDeptAppeal taskListByDeptAppeal,User user);

}
